module Portfolio
  class Commands
    # Show watched stocks and user portfolio
    def show
      # Get list of watched stocks:
      watched_stocks = Store.instance.watched_stocks
      
      # Get stock prices:
      watched_stocks.map! do |stock|
        {
          symbol: stock,
          price: StockPrices.instance.get_price_formatted(stock),
        }
      end
      
      # Get portfolio:
      portfolio = Formatter.instance.format_portfolio(Store.instance.portfolio)
      
      # Output watched stock table:
      Formatter.instance.output_table("Watched Stocks", [:symbol, :price], watched_stocks)
      
      # Output portfolio:
      Formatter.instance.output_table("Portfolio",
        [:symbol, :current_price, :number_owned, :price_paid, :commission, :profit],
        portfolio,
        ['Symbol', 'Current Price', 'Number Owned', 'Price Paid', 'Commission', 'Profit'],
      )
    end
    
    # Start watching a stock
    def watch
      if ARGV.length != 2
        return help()
      end
      
      watched_stocks = Store.instance.watched_stocks
      watched_stocks << ARGV[1].upcase
      
      Store.instance.watched_stocks = watched_stocks
    end
    
    # Stop watching a stock
    def ignore
      if ARGV.length != 2
        return help()
      end
      
      stock_to_remove = ARGV[1].upcase
      watched_stocks = Store.instance.watched_stocks
      watched_stocks.delete stock_to_remove
      
      Store.instance.watched_stocks = watched_stocks
    end
    
    def invest
      if ARGV.length != 5
        return help()
      end
      
      # Get parameters:
      stock = ARGV[1]
      amount_bought = ARGV[2]
      price_paid = ARGV[3]
      commission_paid = ARGV[4]
      
      # Store new investment:
      portfolio = Store.instance.portfolio
      portfolio << {
        stock: stock,
        amount_bought: amount_bought,
        price_paid: price_paid,
        commission_paid: commission_paid,
      }
      Store.instance.portfolio = portfolio
      
      # Tell the user that the investment has been logged and wish the user
      # good luck.
      puts "Investment in #{stock} logged - good luck!"
    end
    
    # Remove stocks from portfolio
    # 
    # FEATURE: Would be nice to be able to have multiple stocks from the same
    #          company, bought at different times, and be able to pick which one
    #          you're selling - rather than just automatically selling them all.
    def sell
      if ARGV.length != 2
        return help()
      end
      
      stock_to_remove = ARGV[1].upcase
      portfolio = Store.instance.portfolio
      portfolio.reject! do |investment|
        investment['stock'].upcase == stock_to_remove
      end
      
      Store.instance.portfolio = portfolio
    end
    
    # Show usage information
    def help
      puts "Portfolio, v#{Portfolio::VERSION}. Commands:\n".bold.underline
      puts "Start watching a stock:".blue
      puts "  " + "--watch <stock>\n".underline
      puts "Stop watching a stock:".blue
      puts "  " + "--ignore <stock>\n".underline
      puts "Add investment to portfolio:".blue
      puts "  " + "--invest <stock> <amount bought> <price paid per stock> <total commission paid>\n".underline
      puts "Remove investment from portfolio:".blue
      puts "  " + "--sell <stock>\n".underline
    end
  end
end
