require 'singleton'
require 'stock_quote'

module Portfolio
  class StockPrices
    include Singleton
    
    def get_stock(symbol)
      StockQuote::Stock.quote(symbol)
    end
    
    def get_price_formatted(symbol)
      # Lookup stock price:
      stock = get_stock(symbol)
      
      # If the response code isn't 200, something went wrong:
      if stock.response_code != 200
        "ERROR".red
      else
        # Get the current stock price:
        price = stock.ask.to_s
        
        # If it's up, show it in green:
        if stock.change > 0.0
          price.green
        # If it's down, show it in red:
        elsif stock.change < 0.0
          return price.red
        # Otherwise, show it normally:
        else
          price
        end
      end
    end
  end
end
