require 'singleton'
require 'json'

module Portfolio
  class Store
    include Singleton
    
    def watched_stocks
      datastore = get_datastore
      datastore['watched_stocks']
    end
    
    def watched_stocks=(watched_stocks)
      data = get_datastore()
      data['watched_stocks'] = watched_stocks
      save_datastore(data)
    end
    
    def portfolio
      datastore = get_datastore
      datastore['portfolio']
    end
    
    def portfolio=(portfolio)
      data = get_datastore()
      data['portfolio'] = portfolio
      save_datastore(data)
    end
    
    protected
    
    def get_datastore
      if @datastore
        @datastore
      else
        begin
          datastore_file_str = File.read(datastore_location)
          @datastore = JSON.parse(datastore_file_str)
        rescue
          create_datastore
          get_datastore
        end
      end
    end
    
    def save_datastore(data)
      File.open(datastore_location, 'w') do |f|
        f.write(data.to_json)
      end
    end
    
    def datastore_location
      File.expand_path('~/.portfolio/datastore.json')
    end
    
    def create_datastore
      begin
        Dir.mkdir File.expand_path('~/.portfolio')
      rescue
      end
      
      File.open(datastore_location, 'w') do |f|
        datastore_hash = {
          watched_stocks: [],
          portfolio: [],
        }
        
        f.write(datastore_hash.to_json)
      end
    end
  end
end
